@@ -13,10 +13,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-char cvsroot_mzscheme_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.11 2004/10/05 22:43:41 wuzzeb Exp $";
+char cvsroot_mzscheme_cxx[] = "$Header: /cvs/projects/SWIG/Source/Modules/mzscheme.cxx,v 1.2 2002/12/11 22:39:28 cheetah Exp $";
 
 /***********************************************************************
- * $Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.11 2004/10/05 22:43:41 wuzzeb Exp $
+ * $Header: /cvs/projects/SWIG/Source/Modules/mzscheme.cxx,v 1.2 2002/12/11 22:39:28 cheetah Exp $
  *
  * mzscheme.cxx
  *
@@ -25,24 +25,23 @@ char cvsroot_mzscheme_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/mzsche
 
 #include "swigmod.h"
 
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
+
 #include <ctype.h>
 
 static const char *usage = (char*)"\
+\n\
 Mzscheme Options (available with -mzscheme)\n\
-     -prefix <name>  - Set a prefix <name> to be prepended to all names\n\
+     -ldflags        - Print runtime libraries to link with\n\
+     -prefix name    - Set a prefix to be appended to all names\n\
      -declaremodule  - Create extension that declares a module\n\
-     -noinit         - Do not emit scheme_initialize, scheme_reload,\n\
-                       scheme_module_name functions\n";
-
-static String     *fieldnames_tab = 0;
-static String     *convert_tab = 0;
-static String     *convert_proto_tab = 0;
-static String     *struct_name = 0;
-static String     *mangled_struct_name = 0;
+\n"
+;
 
 static char *prefix=0;
 static bool declaremodule = false;
-static bool noinit = false;
 static String *module=0;
 static char *mzscheme_path=(char*)"mzscheme";
 static String *init_func_def = 0;
@@ -52,11 +51,6 @@ static  File         *f_header = 0;
 static  File         *f_wrappers = 0;
 static  File         *f_init = 0;
 
-// Used for garbage collection
-static int     exporting_destructor = 0;
-static String *swigtype_ptr = 0;
-static String *cls_swigtype = 0;
-
 class MZSCHEME : public Language {
 public:
 
@@ -89,9 +83,9 @@ public:
 	} else if (strcmp (argv[i], "-declaremodule") == 0) {
 		declaremodule = true;
 		Swig_mark_arg (i);
-	} else if (strcmp (argv[i], "-noinit") == 0) {
-	  noinit = true;
-	  Swig_mark_arg (i);
+	} else if (strcmp (argv[i], "-ldflags") == 0) {
+	  printf("%s\n", SWIG_MZSCHEME_RUNTIME);
+	  SWIG_exit (EXIT_SUCCESS);
 	}
       }
     }
@@ -115,7 +109,7 @@ public:
     SWIG_typemap_lang("mzscheme");
 
     // Read in default typemaps */
-    SWIG_config_file("mzscheme.swg");
+    SWIG_config_file("mzscheme.i");
     allow_overloading();
 
   }
@@ -149,19 +143,21 @@ public:
     Printf(f_runtime, "/* -*- buffer-read-only: t -*- vi: set ro: */\n");
     Swig_banner (f_runtime);
     
+    if (NoInclude) {
+      Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
+    }
+    
     module = Getattr(n,"name");
     
     Language::top(n);
     
     SwigType_emit_type_table (f_runtime, f_wrappers);
-    if (!noinit) {
     Printf(f_init, "Scheme_Object *scheme_reload(Scheme_Env *env) {\n");
-    if (declaremodule) {
-      Printf(f_init, "\tScheme_Env *menv = scheme_primitive_module(scheme_intern_symbol(\"%s\"), env);\n", module);
-    }
-    else {
-      Printf(f_init, "\tScheme_Env *menv = env;\n");
-    }
+	Printf(f_init, "\tScheme_Env *menv = env;\n");
+	if (declaremodule) {
+		Printf(f_init, "\tmenv = scheme_primitive_module(scheme_intern_symbol(\"%s\"), env);\n", module);
+	}
+    Printf (f_init, "\tSWIG_RegisterTypes(swig_types, swig_types_initial);\n");
     Printf(f_init, "%s\n", Char(init_func_def));
 	if (declaremodule) {
 		Printf(f_init, "\tscheme_finish_primitive_module(menv);\n");
@@ -179,7 +175,6 @@ public:
 		Printf(f_init,"   return scheme_make_symbol((char*)\"%s\");\n", module);
 	}
     Printf(f_init,"}\n");
-    }
 
     /* Close all of the files */
     Dump(f_header,f_runtime);
@@ -273,6 +268,7 @@ public:
     numreq  = emit_num_required(l);
     
     // adds local variables
+    Wrapper_add_local(f, "_len", "int _len");
     Wrapper_add_local(f, "lenv", "int lenv = 1");
     Wrapper_add_local(f, "values", "Scheme_Object *values[MAXVALUES]");
     
@@ -368,10 +364,6 @@ public:
       Replaceall(tm,"$source","result");
       Replaceall(tm,"$target","values[0]");
       Replaceall(tm,"$result","values[0]");
-      if (Getattr(n, "feature:new"))
-        Replaceall(tm, "$owner", "1");
-      else
-        Replaceall(tm, "$owner", "0");
       Printv(f->code, tm, "\n",NIL);
     } else {
       throw_unhandled_mzscheme_type_error (d);
@@ -401,7 +393,7 @@ public:
     
     // Wrap things up (in a manner of speaking)
     
-    Printv(f->code, tab4, "return SWIG_MzScheme_PackageValues(lenv, values);\n", NIL);
+    Printv(f->code, tab4, "return swig_package_values(lenv, values);\n", NIL);
     Printf(f->code, "#undef FUNC_NAME\n");
     Printv(f->code, "}\n",NIL);
     
@@ -412,12 +404,9 @@ public:
       // Now register the function
       char temp[256];
       sprintf(temp, "%d", numargs);
-      if (exporting_destructor) {
-        Printf(init_func_def, "SWIG_TypeClientData(SWIGTYPE%s, (void *) %s);\n", swigtype_ptr, wname);
-      } else {
-        Printf(init_func_def, "scheme_add_global(\"%s\", scheme_make_prim_w_arity(%s,\"%s\",%d,%d),menv);\n",
-	       proc_name, wname, proc_name, numreq, numargs);
-      }
+      Printf(init_func_def, "scheme_add_global(\"%s\", scheme_make_prim_w_arity(%s,\"%s\",%d,%d),menv);\n",
+	     proc_name, wname, proc_name, numreq, numargs);
+
     } else {
       if (!Getattr(n,"sym:nextSibling")) {
 	/* Emit overloading dispatch function */
@@ -579,7 +568,7 @@ public:
     
     // Make a static variable;
     
-    Printf (var_name, "_wrap_const_%s", Swig_name_mangle(Getattr(n,"sym:name")));
+    Printf (var_name, "_wrap_const_%s", Swig_name_mangle(iname));
     
     // Build the name for scheme.
     Printv(proc_name, iname,NIL);
@@ -640,136 +629,6 @@ public:
     return SWIG_OK;
   }
 
-  virtual int destructorHandler(Node *n) {
-    exporting_destructor = true;
-    Language::destructorHandler(n);
-    exporting_destructor = false;
-    return SWIG_OK;
-  }
-
-  /* ------------------------------------------------------------
-   * classHandler()
-   * ------------------------------------------------------------ */
-  virtual int classHandler(Node *n) {
-      String     *mangled_classname = 0;
-      String     *real_classname = 0;
-      String     *scm_structname = NewString("");
-      SwigType   *ctype_ptr = NewStringf("p.%s", Getattr(n, "classtype"));
-      
-      SwigType *t = NewStringf("p.%s", Getattr(n, "name"));
-      swigtype_ptr = SwigType_manglestr(t);
-      Delete(t);
-
-      cls_swigtype = SwigType_manglestr(Getattr(n, "name"));
-
-
-      fieldnames_tab       = NewString("");
-      convert_tab          = NewString("");
-      convert_proto_tab    = NewString("");
-
-      struct_name = Getattr(n,"sym:name");
-      mangled_struct_name = Swig_name_mangle(Getattr(n,"sym:name"));
-
-      Printv(scm_structname, struct_name, NIL);
-      Replaceall(scm_structname, "_", "-");
-      
-      real_classname = Getattr(n,"name");
-      mangled_classname = Swig_name_mangle(real_classname);
-
-      Printv(fieldnames_tab, "static const char *_swig_struct_", 
-             cls_swigtype, "_field_names[] = { \n", NIL);
-
-      Printv(convert_proto_tab, "static Scheme_Object *_swig_convert_struct_",
-             cls_swigtype, "(", SwigType_str(ctype_ptr, "ptr"), ");\n", NIL);
-      
-      Printv(convert_tab, "static Scheme_Object *_swig_convert_struct_",
-             cls_swigtype, "(", SwigType_str(ctype_ptr, "ptr"), ")\n {\n", 
-             NIL);
-
-      Printv(convert_tab, 
-             tab4, "Scheme_Object *obj;\n",
-             tab4, "Scheme_Object *fields[_swig_struct_", cls_swigtype, 
-             "_field_names_cnt];\n", 
-             tab4, "int i = 0;\n\n", NIL);
-
-      /* Generate normal wrappers */
-      Language::classHandler(n);
-
-      Printv(convert_tab, tab4, "obj = scheme_make_struct_instance(",
-             "_swig_struct_type_", cls_swigtype, ", i, fields);\n", NIL);
-      Printv(convert_tab, tab4, "return obj;\n}\n\n", NIL);
-
-      Printv(fieldnames_tab, "};\n", NIL);
-
-      Printv(f_header, "static Scheme_Object *_swig_struct_type_", 
-             cls_swigtype, ";\n", NIL);
-
-      Printv(f_header, fieldnames_tab, NIL);
-      Printv(f_header, "#define  _swig_struct_", cls_swigtype, 
-             "_field_names_cnt (sizeof(_swig_struct_", cls_swigtype, 
-             "_field_names)/sizeof(char*))\n", NIL);
-
-      Printv(f_header, convert_proto_tab, NIL);
-      Printv(f_wrappers, convert_tab, NIL);
-
-      Printv(init_func_def, "_swig_struct_type_", cls_swigtype,
-             " = SWIG_MzScheme_new_scheme_struct(menv, \"", scm_structname, "\", ", 
-             "_swig_struct_",  cls_swigtype, "_field_names_cnt,",
-             "(char**) _swig_struct_", cls_swigtype, "_field_names);\n", 
-             NIL);
-      
-      Delete(mangled_classname);
-      Delete(swigtype_ptr);
-      swigtype_ptr = 0;
-      Delete(fieldnames_tab);
-      Delete(convert_tab);
-      Delete(ctype_ptr);
-      Delete(convert_proto_tab);
-      struct_name = 0;
-      mangled_struct_name = 0;
-      Delete(cls_swigtype);
-      cls_swigtype = 0;
-
-      return SWIG_OK;
-  }
-    
-    /* ------------------------------------------------------------
-     * membervariableHandler()
-     * ------------------------------------------------------------ */
-
-    virtual int membervariableHandler(Node *n) {
-        Language::membervariableHandler(n);
-
-	if (!is_smart_pointer()) {
-	  String   *symname    = Getattr(n, "sym:name");
-	  String   *name       = Getattr(n, "name");
-	  SwigType *type       = Getattr(n, "type");
-	  String   *swigtype   = SwigType_manglestr(Getattr(n, "type"));
-	  String   *tm         = 0;
-	  String   *access_mem = NewString("");
-	  SwigType *ctype_ptr  = NewStringf("p.%s", Getattr(n, "type"));
-
-	  Printv(fieldnames_tab, tab4, "\"", symname, "\",\n", NIL);
-	  Printv(access_mem, "(ptr)->", name, NIL);
-	  if ((SwigType_type(type) == T_USER) && (!is_a_pointer(type))) {
-            Printv(convert_tab, tab4, "fields[i++] = ", NIL);
-            Printv(convert_tab, "_swig_convert_struct_", swigtype, 
-                   "((",  SwigType_str(ctype_ptr, ""), ")&((ptr)->", 
-                   name, "));\n", NIL); 
-	  } else if ((tm = Swig_typemap_lookup_new("varout",n,access_mem,0))) {
-            Replaceall(tm,"$result","fields[i++]");
-            Printv(convert_tab, tm, "\n", NIL);
-	  } else
-            Swig_warning(WARN_TYPEMAP_VAR_UNDEF, input_file, line_number,
-                         "Unsupported member variable type %s (ignored).\n", 
-                         SwigType_str(type,0));
-	  
-	  Delete(access_mem);
-	}
-        return SWIG_OK;
-    }
-
-
   /* ------------------------------------------------------------
    * validIdentifer()
    * ------------------------------------------------------------ */
@@ -806,11 +665,9 @@ public:
  * swig_mzscheme()    - Instantiate module
  * ----------------------------------------------------------------------------- */
 
-static Language * new_swig_mzscheme() {
+extern "C" Language *
+swig_mzscheme(void) {
   return new MZSCHEME();
 }
-extern "C" Language * swig_mzscheme(void) {
-  return new_swig_mzscheme();
-}
 
 
