@@ -10,21 +10,18 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_xml_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/xml.cxx,v 1.11 2004/07/12 19:53:23 wsfulton Exp $";
+char cvsroot_xml_cxx[] = "$Header: /cvs/projects/SWIG/Source/Modules/xml.cxx,v 1.4 2003/03/23 14:13:34 beazley Exp $";
 static const char *usage = "\
 XML Options (available with -xml)\n\
-     -xmllang <lang> - Typedef language\n\
-     -xmllite        - More lightweight version of XML\n\
+     -xmllang lang   - Typedef language\n\
      ------\n\
-     deprecated (use -o): -xml <output.xml> - Use <output.xml> as output file (extension .xml mandatory)\n";
+     deprecated (use -o): -xml output.xml - Use output.xml as output file (extension .xml mandatory)\n";
 
 
 #include "swigmod.h"
 
 //static Node *view_top = 0;
 static File *out = 0;
-static int xmllite = 0;
-
 
 class XML
 : public Language
@@ -82,15 +79,7 @@ public:
 			{
 				fputs( usage, stderr );
 			}
-			if( strcmp( argv[iX], "-xmllite" ) == 0 )
-			{
-				Swig_mark_arg (iX);
-                                xmllite = 1;
-                        }
 		}
-
-                // Add a symbol to the parser for conditional compilation
-                Preprocessor_define("SWIGXML 1",0);
 	}
 
   /* Top of the parse tree */
@@ -101,7 +90,6 @@ public:
 		{
 			String *outfile = Getattr(n,"outfile");
 			Replaceall(outfile,".cxx", ".xml");
-			Replaceall(outfile,".cpp", ".xml");
 			Replaceall(outfile,".c", ".xml");
 			out = NewFile(outfile,"w");
 			if (!out)
@@ -112,7 +100,7 @@ public:
 		}
 		Printf( out, "<?xml version=\"1.0\" ?> \n" );
 		Xml_print_tree(n);
-		return SWIG_OK;
+    	return SWIG_OK;
   	}
 
 
@@ -146,10 +134,10 @@ public:
 		print_indent(0);
 		Printf( out, "<attributelist id=\"%ld\" addr=\"%x\" >\n", ++id, obj );
 		indent_level += 4;
-		Iterator ki;
-		ki = First(obj);
-		while (ki.key) {
-		  k = ki.key;
+
+		k = Firstkey(obj);
+		while (k)
+		{
 			if ((Cmp(k,"nodeType") == 0)
 			|| (Cmp(k,"firstChild") == 0)
 			|| (Cmp(k,"lastChild") == 0)
@@ -168,11 +156,11 @@ public:
 			{
 				Xml_print_baselist( Getattr(obj,k) );
 			}
-			else if (!xmllite && Cmp(k,"typescope") == 0)
+			else if (Cmp(k,"typescope") == 0)
 			{
 				Xml_print_typescope( Getattr(obj,k) );
 			}
-			else if (!xmllite && Cmp(k,"typetab") == 0)
+			else if (Cmp(k,"typetab") == 0)
 			{
 				Xml_print_typetab( Getattr(obj,k) );
 			}
@@ -198,8 +186,7 @@ public:
 					Replaceall( o, "<", "&lt;" );
 					Replaceall( o, "\"", "&quot;" );
 					Replaceall( o, "\\", "\\\\" );
-                                        Replaceall( o, "\n", "&#10;" );
-                                        Printf(out,"<attribute name=\"%s\" value=\"%s\" id=\"%ld\" addr=\"%x\" />\n", ck, o, ++id, o );
+					Printf(out,"<attribute name=\"%s\" value=\"%s\" id=\"%ld\" addr=\"%x\" />\n", ck, o, ++id, o );
 					Delete(o);
 					Delete(ck);
 				}
@@ -208,11 +195,11 @@ public:
 					o = Getattr(obj,k);
 					String *ck = NewString(k);
 					Replaceall( ck, ":", "_" );
-                                        Printf(out,"<attribute name=\"%s\" value=\"%x\" id=\"%ld\" addr=\"%x\" />\n", ck, o, ++id, o );
+					Printf(out,"<attribute name=\"%s\" value=\"%x\" id=\"%ld\" addr=\"%x\" />\n", ck, o, ++id, o );
 					Delete(ck);
 				}
 			}
-			ki = Next(ki);
+			k = Nextkey(obj);
 		}
 		indent_level -= 4;
 		print_indent(0);
@@ -271,11 +258,11 @@ public:
 		print_indent(0);
 		Printf( out, "<baselist id=\"%ld\" addr=\"%x\" >\n", ++id, p );
 		indent_level += 4;
-		Iterator s;
-		for (s = First(p); s.item; s = Next(s))
+		String *s;
+		for (s = Firstitem(p); s; s = Nextitem(p))
 		{
 			print_indent(0);
-			Printf( out, "<base name=\"%s\" id=\"%ld\" addr=\"%x\" />\n", s.item, ++id, s.item );
+			Printf( out, "<base name=\"%s\" id=\"%ld\" addr=\"%x\" />\n", s, ++id, s );
 		}
 		indent_level -= 4;
 		print_indent(0);
@@ -314,15 +301,16 @@ public:
 		Printf( out, "<%s id=\"%ld\" addr=\"%x\" >\n", markup, ++id, p );
 		Xml_print_attributes( p );
 		indent_level += 4;
-		Iterator n = First(p);
-		while(n.key)
+		Node * n = Firstitem( p );
+		while(n)
 		{
 			print_indent(0);
-			Printf( out, "<%ssitem id=\"%ld\" addr=\"%x\" >\n", markup, ++id, n.item );
-			Xml_print_attributes( n.item );
-			print_indent(0);
+			Printf( out, "<%ssitem id=\"%ld\" addr=\"%x\" >\n", markup, ++id, n );
+			Xml_print_attributes( n );
 			Printf( out, "</%ssitem >\n", markup );
-			n = Next(n);
+			print_indent(0);
+			Printf( out, " />\n" );
+			n = Nextkey(p);
 		}
 		indent_level -= 4;
 		print_indent(0);
@@ -331,38 +319,11 @@ public:
 
 };
 
-/* -----------------------------------------------------------------------------
- * Swig_print_xml
- *
- * Dump an XML version of the parse tree.  This is different from using the -xml
- * language module normally as it allows the real language module to process the
- * tree first, possibly stuffing in new attributes, so the XML that is output ends
- * up being a post-processing version of the tree.
- * ----------------------------------------------------------------------------- */
 
-void Swig_print_xml(DOH *obj, String* filename)
+extern "C"
 {
-    XML xml;
-    xmllite = 1;
-
-    if (! filename) {
-        out = stdout;
-    }
-    else {
-        out = NewFile(filename, "w");
-        if (!out) {
-            Printf(stderr,"*** Can't open '%s'\n", filename);
-            SWIG_exit(EXIT_FAILURE);
-        }
-    }
-    
-    Printf( out, "<?xml version=\"1.0\" ?> \n" );
-    xml.Xml_print_tree(obj);
-}
-
-static Language * new_swig_xml() {
-        return new XML();
-}
-extern "C" Language * swig_xml( void ) {
-        return new_swig_xml();
+	Language * swig_xml( void )
+	{
+		return new XML();
+	}
 }
