@@ -13,7 +13,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/cwrap.c,v 1.11.2.1.2.40 2002/09/27 17:38:54 beazley Exp $";
+char cvsroot_cwrap_c[] = "$Header: /cvs/projects/SWIG/Source/Swig/cwrap.c,v 1.24 2003/03/21 17:10:52 beazley Exp $";
 
 #include "swig.h"
 
@@ -341,13 +341,17 @@ Swig_cconstructor_call(String_or_char *name) {
  * ----------------------------------------------------------------------------- */
 
 String *
-Swig_cppconstructor_call(String_or_char *name, ParmList *parms) {
+Swig_cppconstructor_base_call(String_or_char *name, ParmList *parms, int skip_self, int disown) {
   String *func;
   String *nname;
   int i = 0;
   int comma = 0;
   Parm *p = parms;
   SwigType *pt;
+  if (skip_self) {
+    if (p) p = nextSibling(p);
+    i++;
+  }
   nname = SwigType_namestr(name);
   func = NewString("");
   Printf(func,"new %s(", nname);
@@ -363,11 +367,29 @@ Swig_cppconstructor_call(String_or_char *name, ParmList *parms) {
     }
     p = nextSibling(p);
   }
+  if (disown >= 0) {
+    if (comma) Printf(func, ",");
+    Printf(func, "%d", disown);
+  }
   Printf(func,")");
   Delete(nname);
   return func;
 }
 
+String *
+Swig_cppconstructor_call(String_or_char *name, ParmList *parms) {
+  return Swig_cppconstructor_base_call(name, parms, 0, -1);
+}
+
+String *
+Swig_cppconstructor_nodirector_call(String_or_char *name, ParmList *parms) {
+  return Swig_cppconstructor_base_call(name, parms, 1, -1);
+}
+
+String *
+Swig_cppconstructor_director_call(String_or_char *name, ParmList *parms) {
+  return Swig_cppconstructor_base_call(name, parms, 0, 0);
+}
 
 /* -----------------------------------------------------------------------------
  * Swig_cdestructor_call()
@@ -473,7 +495,7 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
   name      = Getattr(n,"name");
   qualifier = Getattr(n,"qualifier");
   parms     = CopyParmList(nonvoid_parms(Getattr(n,"parms")));
-  
+ 
   type = NewString(classname);
   if (qualifier) {
     SwigType_push(type,qualifier);
@@ -524,6 +546,62 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
   return SWIG_OK;
 }
 
+/* -----------------------------------------------------------------------------
+ * Swig_methodclass()
+ *
+ * This function returns the class node for a given method or class.
+ * ----------------------------------------------------------------------------- */
+
+Node*
+Swig_methodclass(Node *n) {
+  Node* type = Getattr(n, "nodeType");
+  if (!Cmp(type, "class")) return n;
+  return Getattr(n, "parentNode");
+}
+
+int
+Swig_directorbase(Node *n) {
+  Node *classNode = Swig_methodclass(n);
+  return (classNode && (Getattr(classNode, "directorBase") != 0));
+}
+
+int
+Swig_directorclass(Node *n) {
+  Node *classNode = Swig_methodclass(n);
+  return (Getattr(classNode, "vtable") != 0);
+}
+
+int
+Swig_directormethod(Node *n) {
+  Node *classNode = Swig_methodclass(n);
+  if (classNode) {
+    Node *vtable = Getattr(classNode, "vtable");
+    if (vtable) {
+      String *name = Getattr(n, "name");
+      String *decl = Getattr(n, "decl");
+      String *method_id = NewStringf("%s|%s", name, decl);
+      Hash *item = Getattr(vtable, method_id);
+      Delete(method_id);
+      if (item) {
+        return (Getattr(item, "director") != 0);
+      }
+    }
+  }
+  return 0;
+}
+
+Node *
+Swig_directormap(Node *module, String *type) {
+  int is_void = !Cmp(type, "void");
+  if (!is_void && module) {
+    String* base = SwigType_base(type);
+    Node *directormap = Getattr(module, "wrap:directormap");
+    if (directormap) return Getattr(directormap, base);
+  }
+  return 0;
+}
+
+	
 /* -----------------------------------------------------------------------------
  * Swig_ConstructorToFunction()
  *
@@ -531,24 +609,35 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
  * ----------------------------------------------------------------------------- */
 
 int
-Swig_ConstructorToFunction(Node *n, String *classname, int cplus, int flags)
+Swig_ConstructorToFunction(Node *n, String *classname, 
+			   String *none_comparison, int cplus, int flags)
 {
   ParmList *parms;
   SwigType *type;
   String   *membername;
   String   *mangled;
+  Node     *classNode;
+  int      use_director;
+  
+  classNode = Swig_methodclass(n);
+  use_director = Swig_directorclass(n);
+  
   membername = Swig_name_construct(classname);
   mangled = Swig_name_mangle(membername);
 
   parms = CopyParmList(nonvoid_parms(Getattr(n,"parms")));
+
   type  = NewString(classname);
   SwigType_add_pointer(type);
 
   if (flags & CWRAP_EXTEND) {
     String *code = Getattr(n,"code");
     if (code) {
-      String *wrap;
-      String *s = NewStringf("%s(%s)", mangled, ParmList_str(parms));
+      String *wrap, *s;
+      if (Getattr(n,"sym:overloaded") && code) {
+	Append(mangled,Getattr(n,"sym:overname"));
+      }
+      s = NewStringf("%s(%s)", mangled, ParmList_str(parms));
       wrap = SwigType_str(type,s);
       Delete(s);
       Printv(wrap,code,"\n",NIL);
@@ -558,7 +647,33 @@ Swig_ConstructorToFunction(Node *n, String *classname, int cplus, int flags)
     Setattr(n,"wrap:action", Swig_cresult(type,"result", Swig_cfunction_call(mangled,parms)));
   } else {
     if (cplus) {
-      Setattr(n,"wrap:action", Swig_cresult(type,"result", Swig_cppconstructor_call(classname,parms)));
+      /* if a C++ director class exists, create it rather than the original class */
+      if (use_director) {
+	Node *parent = Swig_methodclass(n);
+	String *name = Getattr(parent, "sym:name");
+        String* directorname = NewStringf("__DIRECTOR__%s", name);
+	String* action = NewString("");
+	String* tmp_none_comparison = Copy(none_comparison);
+	
+	Replaceall( tmp_none_comparison, "$arg", "arg1" );
+
+        /* if Python class has been subclassed, create a director instance.  
+	 * otherwise, just create a normal instance.
+         */
+	/* arty: arg1 != Py_None => tmp_none_comparison */
+	Printv(action, "if (",tmp_none_comparison,") {/* subclassed */\n",
+	               Swig_cresult(type, "result", Swig_cppconstructor_director_call(directorname, parms)),
+		       "} else {\n",
+	               Swig_cresult(type, "result", Swig_cppconstructor_nodirector_call(classname, parms)),
+		       "}\n",
+		       NULL);
+	Setattr(n, "wrap:action", action);
+	Delete(tmp_none_comparison);
+	Delete(action);
+        Delete(directorname);
+      } else {
+        Setattr(n,"wrap:action", Swig_cresult(type,"result", Swig_cppconstructor_call(classname,parms)));
+      }
     } else {
       Setattr(n,"wrap:action", Swig_cresult(type,"result", Swig_cconstructor_call(classname)));
     }
@@ -583,7 +698,7 @@ Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags)
 {
   SwigType *type;
   Parm     *p;
-
+ 
   type  = NewString(classname);
   SwigType_add_pointer(type);
   p = NewParm(type,"self");
@@ -606,7 +721,9 @@ Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags)
     Delete(mangled);
   } else {
     if (cplus) {
-      Setattr(n,"wrap:action", NewStringf("%s;\n", Swig_cppdestructor_call()));
+      String* action = NewString("");
+      Printf(action, "%s;\n", Swig_cppdestructor_call());
+      Setattr(n,"wrap:action", action);
     } else {
       Setattr(n,"wrap:action", NewStringf("%s;\n", Swig_cdestructor_call()));
     }
