@@ -9,7 +9,7 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.	
  *
- * $Header: /cvs/projects/SWIG/Source/Swig/swig.h,v 1.51.2.3.2.14 2001/09/19 13:26:49 beazley Exp $
+ * $Header: /cvs/projects/SWIG/Source/Swig/swig.h,v 1.69 2003/03/08 03:25:09 arty Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef _SWIGCORE_H
@@ -22,6 +22,12 @@
 
 #include "doh.h"
 
+/* Status codes */
+
+#define SWIG_OK         1
+#define SWIG_ERROR      0
+#define SWIG_NOWRAP     0
+
 /* Short names for common data types */
 
 typedef  DOH     String;
@@ -33,12 +39,14 @@ typedef  DOH     Parm;
 typedef  DOH     ParmList;
 typedef  DOH     Node;
 typedef  DOH     Symtab;
+typedef  DOH     Typetab;
+typedef  DOH     SwigType;
 
 /* --- Legacy DataType interface.  These type codes are provided solely 
        for backwards compatibility with older modules --- */
 
 /* --- The ordering of type values is used to determine type-promotion 
-   in the parser.  Do not change */
+       in the parser.  Do not change */
 
 /* Numeric types */
 
@@ -69,6 +77,7 @@ typedef  DOH     Symtab;
 #define   T_ARRAY      36
 #define   T_FUNCTION   37
 #define   T_MPOINTER   38
+#define   T_VARARGS    39
 #define   T_SYMBOL     98
 #define   T_ERROR      99
 
@@ -81,15 +90,16 @@ extern FILE    *Swig_open(const String_or_char *name);
 extern String  *Swig_read_file(FILE *f);
 extern String  *Swig_include(const String_or_char *name);
 extern int      Swig_insert_file(const String_or_char *name, File *outfile);
-extern int      Swig_bytes_read();
 extern void     Swig_set_config_file(const String_or_char *filename);
 extern String  *Swig_get_config_file(void);
 extern void     Swig_swiglib_set(const String_or_char *);
 extern String  *Swig_swiglib_get();
 extern void     Swig_register_filebyname(const String_or_char *filename, File *outfile);
 extern File    *Swig_filebyname(const String_or_char *filename);
-
-#define OUTFILE(x)   Swig_filebyname(x)
+extern char    *Swig_file_suffix(const String_or_char *filename);
+extern char    *Swig_file_basename(const String_or_char *filename);
+extern char    *Swig_file_filename(const String_or_char *filename);
+extern char    *Swig_file_dirname(const String_or_char *filename);
 
 #ifdef MACSWIG
 #  define SWIG_FILE_DELIMETER  ":"
@@ -172,24 +182,29 @@ extern void         SwigScanner_idstart(SwigScanner *, char *idchar);
 #define   SWIG_TOKEN_DOLLAR       46
 #define   SWIG_TOKEN_CODEBLOCK    47
 #define   SWIG_TOKEN_RSTRING      48
+#define   SWIG_TOKEN_LONGLONG     49
+#define   SWIG_TOKEN_ULONGLONG    50
 #define   SWIG_TOKEN_ILLEGAL      98
 #define   SWIG_TOKEN_LAST         99 
 
 /* --- Functions for manipulating the string-based type encoding --- */
-typedef DOH SwigType;
 
 extern SwigType   *NewSwigType(int typecode);
 extern void        SwigType_add_pointer(SwigType *t);
 extern void        SwigType_add_memberpointer(SwigType *t, String_or_char *qual);
 extern void        SwigType_del_pointer(SwigType *t);
 extern void        SwigType_add_array(SwigType *t, String_or_char *size);
+extern SwigType   *SwigType_pop_arrays(SwigType *t);
 extern void        SwigType_add_reference(SwigType *t);
 extern void        SwigType_add_qualifier(SwigType *t, String_or_char *qual);
 extern void        SwigType_add_function(SwigType *t, ParmList *parms);
+extern void        SwigType_add_template(SwigType *t, ParmList *parms);
+extern SwigType   *SwigType_pop_function(SwigType *t);
+extern ParmList   *SwigType_function_parms(SwigType *t);
 extern List       *SwigType_split(SwigType *t);
 extern String     *SwigType_pop(SwigType *t);
 extern void        SwigType_push(SwigType *t, SwigType *s);
-extern List       *SwigType_parmlist(SwigType *p);
+extern List       *SwigType_parmlist(const SwigType *p);
 extern String     *SwigType_parm(String *p);
 extern String     *SwigType_str(SwigType *s, const String_or_char *id);
 extern String     *SwigType_lstr(SwigType *s, const String_or_char *id);
@@ -198,58 +213,88 @@ extern String     *SwigType_lcaststr(SwigType *s, const String_or_char *id);
 extern String     *SwigType_manglestr(SwigType *t);
 extern SwigType   *SwigType_ltype(SwigType *t);
 extern int         SwigType_ispointer(SwigType *t);
+extern int         SwigType_ispointer_return(SwigType *t);
 extern int         SwigType_ismemberpointer(SwigType *t);
 extern int         SwigType_isreference(SwigType *t);
+extern int         SwigType_isreference_return(SwigType *t);
 extern int         SwigType_isarray(SwigType *t);
 extern int         SwigType_isfunction(SwigType *t);
 extern int         SwigType_isqualifier(SwigType *t);
 extern int         SwigType_isconst(SwigType *t);
-
+extern int         SwigType_issimple(SwigType *t);
+extern int         SwigType_ismutable(SwigType *t);
+extern int         SwigType_isvarargs(const SwigType *t);
+extern int         SwigType_istemplate(const SwigType *t);
+extern int         SwigType_isenum(SwigType *t);
+extern int         SwigType_check_decl(SwigType *t, const String_or_char *decl);
 extern SwigType   *SwigType_strip_qualifiers(SwigType *t);
-
 extern String     *SwigType_base(SwigType *t);
+extern String     *SwigType_namestr(const SwigType *t);
+extern String     *SwigType_templateprefix(SwigType *t);
+extern String     *SwigType_templatesuffix(const SwigType *t);
+extern String     *SwigType_templateargs(SwigType *t);
 extern String     *SwigType_prefix(SwigType *t);
-extern void        SwigType_setbase(SwigType *t, String_or_char *name);
-
-extern int         SwigType_typedef(SwigType *type, String_or_char *name);
-extern void        SwigType_inherit(String *subclass, String *baseclass);
-extern void        SwigType_new_scope();
-extern void        SwigType_reset_scopes();
-extern void        SwigType_set_scope_name(String_or_char *name);
-extern void        SwigType_merge_scope(Hash *scope);
-extern Hash       *SwigType_pop_scope();
-extern SwigType   *SwigType_typedef_resolve(SwigType *t);
-extern SwigType   *SwigType_typedef_resolve_all(SwigType *t);
-extern SwigType   *SwigType_typedef_qualified(SwigType *t);
-extern int         SwigType_istypedef(SwigType *t);
-extern int         SwigType_cmp(String_or_char *pat, SwigType *t);
 extern int         SwigType_array_ndim(SwigType *t);
 extern String     *SwigType_array_getdim(SwigType *t, int n);
 extern void        SwigType_array_setdim(SwigType *t, int n, String_or_char *rep);
 extern SwigType   *SwigType_array_type(SwigType *t);
 extern String     *SwigType_default(SwigType *t);
-extern int         SwigType_type(SwigType *t);
+extern void        SwigType_typename_replace(SwigType *t, String *pat, String *rep);
+
+/* --- Type-system managment --- */
+extern void        SwigType_typesystem_init();
+extern int         SwigType_typedef(SwigType *type, String_or_char *name);
+extern int         SwigType_typedef_class(String_or_char *name);
+extern int         SwigType_typedef_using(String_or_char *qname);
+extern void        SwigType_inherit(String *subclass, String *baseclass, String *cast);
+extern int         SwigType_issubtype(SwigType *subtype, SwigType *basetype);
+extern void        SwigType_scope_alias(String *aliasname, Typetab *t);
+extern void        SwigType_using_scope(Typetab *t);
+extern void        SwigType_new_scope(String_or_char *name);
+extern void        SwigType_reset_scopes();
+extern void        SwigType_set_scope_name(String_or_char *name);
+extern void        SwigType_inherit_scope(Typetab *scope);
+extern Typetab    *SwigType_pop_scope();
+extern Typetab    *SwigType_set_scope(Typetab *h);
+extern void        SwigType_print_scope(Typetab *t);
+extern SwigType   *SwigType_typedef_resolve(SwigType *t);
+extern SwigType   *SwigType_typedef_resolve_all(SwigType *t);
+extern SwigType   *SwigType_typedef_qualified(SwigType *t);
+extern int         SwigType_istypedef(SwigType *t);
+extern int         SwigType_isclass(SwigType *t);
+extern void        SwigType_attach_symtab(Symtab *syms);
 extern void        SwigType_remember(SwigType *t);
+extern void        SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata);
+extern void      (*SwigType_remember_trace(void (*tf)(SwigType *, String *, String *)))(SwigType *, String *, String *);
 extern void        SwigType_emit_type_table(File *f_headers, File *f_table);
+extern int         SwigType_type(SwigType *t);
 
 /* --- Symbol table module --- */
 
-extern void    Swig_symbol_init();
-extern void    Swig_symbol_setscopename(String_or_char *name);
-extern String *Swig_symbol_getscopename();
-extern String *Swig_symbol_qualifiedscopename(Symtab *symtab);
-extern Symtab *Swig_symbol_newscope();
-extern Symtab *Swig_symbol_setscope(Symtab *);
-extern Symtab *Swig_symbol_current();
-extern Symtab *Swig_symbol_popscope();
-extern Node   *Swig_symbol_add(String_or_char *symname, Node *node);
-extern Node   *Swig_symbol_add_tag(String_or_char *symname, Node *node);
-extern Node   *Swig_symbol_lookup(String_or_char *symname);
-extern Node   *Swig_symbol_lookup_local(String_or_char *symname);
-extern Node   *Swig_symbol_lookup_tag(String_or_char *symname);
-extern String *Swig_symbol_qualified(Node *node);
-extern Node   *Swig_symbol_isoverloaded(Node *node);
-extern void    Swig_symbol_remove(Node *node);
+extern void     Swig_symbol_init();
+extern void     Swig_symbol_setscopename(const String_or_char *name);
+extern String   *Swig_symbol_getscopename();
+extern String   *Swig_symbol_qualifiedscopename(Symtab *symtab);
+extern Symtab   *Swig_symbol_newscope();
+extern Symtab   *Swig_symbol_setscope(Symtab *);
+extern Symtab   *Swig_symbol_getscope(const String_or_char *symname);
+extern Symtab   *Swig_symbol_current();
+extern Symtab   *Swig_symbol_popscope();
+extern Node     *Swig_symbol_add(String_or_char *symname, Node *node);
+extern void      Swig_symbol_cadd(String_or_char *symname, Node *node);
+extern Node     *Swig_symbol_clookup(String_or_char *symname, Symtab *tab);
+extern Node     *Swig_symbol_clookup_check(String_or_char *symname, Symtab *tab, int (*check)(Node *));
+extern Symtab   *Swig_symbol_cscope(String_or_char *symname, Symtab *tab);
+extern Node     *Swig_symbol_clookup_local(String_or_char *symname, Symtab *tab);
+extern Node     *Swig_symbol_clookup_local_check(String_or_char *symname, Symtab *tab, int (*check)(Node *));
+extern String   *Swig_symbol_qualified(Node *node);
+extern Node     *Swig_symbol_isoverloaded(Node *node);
+extern void      Swig_symbol_remove(Node *node);
+extern void      Swig_symbol_alias(String_or_char *aliasname, Symtab *tab);
+extern void      Swig_symbol_inherit(Symtab *tab);
+extern SwigType *Swig_symbol_type_qualify(SwigType *ty, Symtab *tab);
+extern String   *Swig_symbol_string_qualify(String *s, Symtab *tab);
+extern SwigType *Swig_symbol_typedef_reduce(SwigType *ty, Symtab *tab);
 
 /* --- Parameters and Parameter Lists --- */
 
@@ -275,13 +320,7 @@ extern String     *ParmList_protostr(ParmList *);
 #define  nextSibling(x)            Getattr(x,"nextSibling")
 #define  firstChild(x)             Getattr(x,"firstChild")
 #define  lastChild(x)              Getattr(x,"lastChild")
-
-/* Special accessors for symbol tables */
-
-#define  nextSymbol(x)             Getattr(x,"$symnext")
-#define  prevSymbol(x)             Getattr(x,"$symprev")
-#define  nodeSymtab(x)             Getattr(x,"$symtab")
-#define  nodeSymbol(x)             Getattr(x,"$symname")
+extern int checkAttribute(Node *obj, const String_or_char *name, const String_or_char *value);
 
 /* Macros to set up the DOM tree (mostly used by the parser) */
 
@@ -293,19 +332,24 @@ extern String     *ParmList_protostr(ParmList *);
 #define  set_lastChild(x,v)        Setattr(x,"lastChild",v)
 
 extern void appendChild(Node *node, Node *child);
+extern void deleteNode(Node *node);
+extern Node *copyNode(Node *node);
+
+extern void Swig_tag_nodes(Node *node, const String_or_char *attrname, DOH *value);
+
+extern int  Swig_require(const char *ns, Node *node, ...);
+extern int  Swig_save(const char *ns, Node *node,...);
+extern void Swig_restore(Node *node);
 
 /* Debugging of parse trees */
 extern void Swig_debug_emit(int);
-extern void Swig_dump_tags(DOH *obj, DOH *root);
-extern void Swig_dump_tree(DOH *obj);
-extern void Swig_dump_rules();
+extern void Swig_print_tags(File *obj, Node *root);
+extern void Swig_print_tree(Node *obj);
+extern void Swig_print_node(Node *obj);
 
 /* -- Wrapper function Object */
 
 typedef struct {
-  SwigType  *_type;
-  ParmList  *_parms;
-  String    *_name;
   Hash      *localh;
   String    *def;
   String    *locals;
@@ -314,159 +358,137 @@ typedef struct {
 
 extern Wrapper  *NewWrapper();
 extern void      DelWrapper(Wrapper *w);
+extern void      Wrapper_compact_print_mode_set(int flag);
 extern void      Wrapper_pretty_print(String *str, File *f);
+extern void      Wrapper_compact_print(String *str, File *f);
 extern void      Wrapper_print(Wrapper *w, File *f);
 extern int       Wrapper_add_local(Wrapper *w, const String_or_char *name, const String_or_char *decl);
 extern int       Wrapper_add_localv(Wrapper *w, const String_or_char *name, ...);
 extern int       Wrapper_check_local(Wrapper *w, const String_or_char *name);
 extern char     *Wrapper_new_local(Wrapper *w, const String_or_char *name, const String_or_char *decl);
 extern char     *Wrapper_new_localv(Wrapper *w, const String_or_char *name, ...);
-extern SwigType *Wrapper_Gettype(Wrapper *w);
-extern void      Wrapper_Settype(Wrapper *w, SwigType *t);
-extern ParmList *Wrapper_Getparms(Wrapper *w);
-extern void      Wrapper_Setparms(Wrapper *w, ParmList *l);
-extern char     *Wrapper_Getname(Wrapper *w);
-extern void      Wrapper_Setname(Wrapper *w, String_or_char *name);
 
 /* --- Naming functions --- */
 
-extern void     Swig_name_register(String_or_char *method, String_or_char *format);
-extern String  *Swig_name_mangle(String_or_char *s);
-extern String  *Swig_name_wrapper(String_or_char *fname);
-extern String  *Swig_name_member(String_or_char *classname, String_or_char *mname);
-extern String  *Swig_name_get(String_or_char *vname);
-extern String  *Swig_name_set(String_or_char *vname);
-extern String  *Swig_name_construct(String_or_char *classname);
-extern String  *Swig_name_destroy(String_or_char *classname);
+extern void     Swig_name_register(const String_or_char *method, const String_or_char *format);
+extern void     Swig_name_unregister(const String_or_char *method);
+extern String  *Swig_name_mangle(const String_or_char *s);
+extern String  *Swig_name_wrapper(const String_or_char *fname);
+extern String  *Swig_name_member(const String_or_char *classname, const String_or_char *mname);
+extern String  *Swig_name_get(const String_or_char *vname);
+extern String  *Swig_name_set(const String_or_char *vname);
+extern String  *Swig_name_construct(const String_or_char *classname);
+extern String  *Swig_name_copyconstructor(const String_or_char *classname);
+extern String  *Swig_name_destroy(const String_or_char *classname);
+extern String  *Swig_name_disown(const String_or_char *classname);
 
-/* --- Mapping interface --- */
+/* --- parameterized rename functions --- */
 
-extern void       Swig_map_add(Hash *ruleset, Hash *parms, DOH *obj);
-extern DOH       *Swig_map_match(Hash *ruleset, Hash *parms, int *nmatch);
+extern void      Swig_name_object_set(Hash *namehash, String_or_char *name, SwigType *decl, DOH *object);
+extern DOH      *Swig_name_object_get(Hash *namehash, String_or_char *prefix, String_or_char *name, SwigType *decl);
+extern void      Swig_name_object_inherit(Hash *namehash, String *base, String *derived);
+extern void      Swig_features_get(Hash *features, String_or_char *prefix, String_or_char *name, SwigType *decl, Node *n);
+extern void      Swig_feature_set(Hash *features, String_or_char *name, SwigType *decl, String_or_char *fname, String *value);
 
 /* --- Misc --- */
 extern char      *Swig_copy_string(const char *c);
 extern void       Swig_banner(File *f);
-extern void       Swig_section(File *f, const String_or_char *s);
-extern DOH       *Swig_temp_result(DOH *x);
 extern String    *Swig_string_escape(String *s);
 extern String    *Swig_string_mangle(String *s);
+extern String    *Swig_scopename_prefix(String *s);
+extern String    *Swig_scopename_last(String *s);
+extern String    *Swig_scopename_first(String *s);
+extern String    *Swig_scopename_suffix(String *s);
+extern int        Swig_scopename_check(String *s);
+
 extern void       Swig_init();
 extern void       Swig_warn(const char *filename, int line, const char *msg);
 
+
 #define WARNING(msg) Swig_warn(__FILE__,__LINE__,msg)
 
+extern void       Swig_warning(int num, const String_or_char *filename, int line, const char *fmt, ...);
+extern void       Swig_error(const String_or_char *filename, int line, const char *fmt, ...);
+extern int        Swig_error_count(void);
+extern void       Swig_error_silent(int s);
+extern void       Swig_warnfilter(const String_or_char *wlist, int val);
+extern void       Swig_warnall(void);
+extern int        Swig_warn_count(void);
+
 /* --- C Wrappers --- */
-extern String    *Swig_clocal(SwigType *t, String_or_char *name, String_or_char *value);
-extern SwigType  *Swig_clocal_type(SwigType *t);
-extern String    *Swig_clocal_deref(SwigType *t, String_or_char *name);
-extern String    *Swig_clocal_assign(SwigType *t, String_or_char *name);
 extern String    *Swig_cparm_name(Parm *p, int i);
+extern String    *Swig_clocal(SwigType *t, String_or_char *name, String_or_char *value);
+extern String    *Swig_wrapped_var_type(SwigType *t);
+extern String    *Swig_wrapped_var_deref(SwigType *t, String_or_char *name);
+extern String    *Swig_wrapped_var_assign(SwigType *t, String_or_char *name);
 extern int        Swig_cargs(Wrapper *w, ParmList *l);
-extern void       Swig_cresult(Wrapper *w, SwigType *t, String_or_char *name, String_or_char *decl);
-extern void       Swig_cppresult(Wrapper *w, SwigType *t, String_or_char *name, String_or_char *decl);
+extern String    *Swig_cresult(SwigType *t, const String_or_char *name, const String_or_char *decl);
+
 extern String    *Swig_cfunction_call(String_or_char *name, ParmList *parms);
-extern String    *Swig_cmethod_call(String_or_char *name, ParmList *parms);
+extern String    *Swig_cmethod_call(String_or_char *name, ParmList *parms, String_or_char *self);
 extern String    *Swig_cconstructor_call(String_or_char *name);
 extern String    *Swig_cppconstructor_call(String_or_char *name, ParmList *parms);
 extern String    *Swig_cdestructor_call();
 extern String    *Swig_cppdestructor_call();
-extern String    *Swig_cmemberset_call(String_or_char *name, SwigType *t);
-extern String    *Swig_cmemberget_call(String_or_char *name, SwigType *t);
+extern String    *Swig_cmemberset_call(String_or_char *name, SwigType *type, String_or_char *self);
+extern String    *Swig_cmemberget_call(String_or_char *name, SwigType *t, String_or_char *self);
 
-extern Wrapper   *Swig_cfunction_wrapper(String_or_char *funcname,
-                                         SwigType *rtype,
-                                         ParmList *parms,
-                                         String_or_char *code);
+/* --- Transformations --- */
 
-extern Wrapper   *Swig_cmethod_wrapper(String_or_char *classname,
-				       String_or_char *methodname,
-				       SwigType *rtype,
-				       ParmList *parms,
-				       String_or_char *code);
+extern int        Swig_MethodToFunction(Node *n, String *classname, int flags);
+extern int        Swig_ConstructorToFunction(Node *n, String *classname, 
+					     String *none_comparison,
+					     int cplus, int flags);
+extern int        Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags);
+extern int        Swig_MembersetToFunction(Node *n, String *classname, int flags);
+extern int        Swig_MembergetToFunction(Node *n, String *classname, int flags);
+extern int        Swig_VargetToFunction(Node *n);
+extern int        Swig_VarsetToFunction(Node *n);
 
-extern Wrapper   *Swig_cdestructor_wrapper(String_or_char *classname,
-                                           String_or_char *code);
+#define  CWRAP_EXTEND             0x01
+#define  CWRAP_SMART_POINTER      0x02
 
-extern Wrapper   *Swig_cppdestructor_wrapper(String_or_char *classname,
-					     String_or_char *code);
+/* --- Director Helpers --- */
+extern Node      *Swig_methodclass(Node *n);
+extern int        Swig_directorbase(Node *n);
+extern int        Swig_directorclass(Node *n);
+extern int        Swig_directormethod(Node *n);
+extern Node      *Swig_directormap(Node *n, String *type);
 
-extern Wrapper   *Swig_cconstructor_wrapper(String_or_char *classname,
-                                            ParmList *parms,
-                                            String_or_char *code);
+/* --- Legacy Typemap API (somewhat simplified, ha!) --- */
 
-extern Wrapper   *Swig_cppconstructor_wrapper(String_or_char *classname,
-					      ParmList *parms,
-					      String_or_char *code);
+extern     void   Swig_typemap_init();
+extern     void   Swig_typemap_register(const String_or_char *op, ParmList *pattern, String_or_char *code, ParmList *locals, ParmList *kwargs);
+extern     int    Swig_typemap_copy(const String_or_char *op, ParmList *srcpattern, ParmList *pattern);
+extern     void   Swig_typemap_clear(const String_or_char *op, ParmList *pattern);
+extern     int    Swig_typemap_apply(ParmList *srcpat, ParmList *destpat);
+extern     void   Swig_typemap_clear_apply(ParmList *pattern);
+extern     void   Swig_typemap_debug();
 
+extern Hash  *Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *pname, SwigType **matchtype);
+extern Hash  *Swig_typemap_search_multi(const String_or_char *op, ParmList *parms, int *nmatch);
+extern String *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_char *pname, String_or_char *lname,
+				  String_or_char *source, String_or_char *target, Wrapper *f);
 
-extern Wrapper   *Swig_cmemberset_wrapper(String_or_char *classname,
-                                          String_or_char *membername,
-					  SwigType *type,
-                                          String_or_char *code);
+extern String *Swig_typemap_lookup_new(const String_or_char *op, Node *n, const String_or_char *lname, Wrapper *f);
 
-extern Wrapper   *Swig_cmemberget_wrapper(String_or_char *classname,
-                                          String_or_char *membername,
-					  SwigType *type,
-                                          String_or_char *code);
-
-extern Wrapper   *Swig_cvarset_wrapper(String_or_char *varname,
-				       SwigType *type,
-				       String_or_char *code);
-
-extern Wrapper   *Swig_cvarget_wrapper(String_or_char *varname,
-				       SwigType *type,
-				       String_or_char *code);
-
-
-/* --- Legacy Typemap API (somewhat simplified) --- */
-
-extern void   Swig_typemap_init();
-extern void   Swig_typemap_register(const String_or_char *op, SwigType *type, String_or_char *name, String_or_char *code, ParmList *locals);
-extern void   Swig_typemap_copy(const String_or_char *op, SwigType *stype, String_or_char *sname,
-				SwigType *ttype, String_or_char *tname);
-extern void   Swig_typemap_clear(const String_or_char *op, SwigType *type, String_or_char *name);
-extern void   Swig_typemap_apply(SwigType *tm_type, String_or_char *tmname, SwigType *type, String_or_char *pname);
-extern void   Swig_typemap_clear_apply(SwigType *type, String_or_char *pname);
-extern void   Swig_typemap_debug();
-extern Hash  *Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *pname);
-extern char  *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_char *pname, String_or_char *source, String_or_char *target, Wrapper *f);
+extern String *Swig_typemap_lookup_multi(const String_or_char *op, ParmList *parms, String_or_char *source, Wrapper *f, int *nmatch);
 extern void   Swig_typemap_new_scope();
 extern Hash  *Swig_typemap_pop_scope();
 
-/* --- Legacy %except directive API --- */
-extern void   Swig_except_register(String_or_char *code);
-extern char  *Swig_except_lookup();
-extern void   Swig_except_clear();
+extern void   Swig_typemap_attach_parms(const String_or_char *op, ParmList *parms, Wrapper *f);
 
-/* --- Attribute access macros --- */
-
-#define Gettype(x)         Getattr(x,"type")
-#define Getname(x)         Getattr(x,"name")
-#define Getvalue(x)        Getattr(x,"value")
-#define Getlname(x)        Getattr(x,"lname")
-#define Getignore(x)       GetInt(x,"ignore")
-#define Getparms(x)        Getattr(x,"parms")
-#define Getdecl(x)         Getattr(x,"decl")
-#define Getstorage(x)      Getattr(x,"storage")
-#define Getsymname(x)      Getattr(x,"$symname")
-#define Geterror(x)        Getattr(x,"error")
-
-#define Settype(x,v)       Setattr(x,"type",v)
-#define Setname(x,v)       Setattr(x,"name",v)
-#define Setlname(x,v)      Setattr(x,"lname",v)
-#define Setvalue(x,v)      Setattr(x,"value", v)
-#define Setignore(x,v)     SetInt(x,"ignore",v)
-#define Setparms(x,v)      Setattr(x,"parms", v)
-#define Setdecl(x,d)       Setattr(x,"decl",d)
-#define Setstorage(x,s)    Setattr(x,"storage",s)
-#define Seterror(x,e)      Setattr(x,"error",e)
-#define Setsymname(x,s)    Setattr(x,"$symname",s)
-#define Getnext(x)         Getattr(x,"nextSibling")
+/* --- Code fragment support --- */
 
+extern  void  Swig_fragment_register(String *name, String *section, String *code);
+extern  void  Swig_fragment_emit(String *name);
+  
 #endif
 
 
 
 
 
+
+
+
