@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typesys_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typesys.c,v 1.15 2004/04/09 19:22:01 cheetah Exp $";
+char cvsroot_typesys_c[] = "$Header: /cvs/projects/SWIG/Source/Swig/typesys.c,v 1.4 2003/03/27 23:17:27 beazley Exp $";
 
 #include "swig.h"
 
@@ -104,23 +104,10 @@ static Typetab *global_scope  = 0;                /* The global scope
 static Hash    *scopes        = 0;                /* Hash table containing fully qualified scopes */
 
 /* Performance optimization */
-#define SWIG_TYPEDEF_RESOLVE_CACHE 
 static Hash     *typedef_resolve_cache = 0;
 static Hash     *typedef_all_cache = 0;
 static Hash     *typedef_qualified_cache = 0;
 
-/* common attribute keys, to avoid calling find_key all the times */
-static String *k_name = 0;
-static String *k_qname = 0;
-static String *k_symtab = 0;
-static String *k_using = 0;
-static String *k_scope = 0;
-static String *k_typetab = 0;
-static String *k_inherit = 0;
-static String *k_parent = 0;
-static String *k_value = 0;
-
-
 static void flush_cache() {
   typedef_resolve_cache = 0;
   typedef_all_cache = 0;
@@ -130,32 +117,21 @@ static void flush_cache() {
 /* Initialize the scoping system */
 
 void SwigType_typesystem_init() {
-  k_name = NewString("name");
-  k_qname = NewString("qname");
-  k_symtab = NewString("symtab");
-  k_using = NewString("using");
-  k_scope = NewString("scope");
-  k_typetab = NewString("typetab");
-  k_inherit = NewString("inherit");
-  k_parent = NewString("parent");
-  k_value = NewString("value");
-
   if (global_scope) Delete(global_scope);
   if (scopes)       Delete(scopes);
 
   current_scope = NewHash();
   global_scope  = current_scope;
 
-  Setattr(current_scope,k_name,"");              /* No name for global scope */
+  Setattr(current_scope,"name","");              /* No name for global scope */
   current_typetab = NewHash();
-  Setattr(current_scope,k_typetab, current_typetab);
+  Setattr(current_scope,"typetab", current_typetab);
 
   current_symtab = 0;
   scopes = NewHash();
   Setattr(scopes,"",current_scope);
 }
 
-
 /* -----------------------------------------------------------------------------
  * SwigType_typedef()
  *
@@ -213,15 +189,15 @@ int SwigType_typedef_class(String_or_char *name) {
 
 String *
 SwigType_scope_name(Typetab *ttab) {
-  String *qname = NewString(Getattr(ttab,k_name));
-  ttab = Getattr(ttab,k_parent);
+  String *qname = NewString(Getattr(ttab,"name"));
+  ttab = Getattr(ttab,"parent");
   while (ttab) {
-    String *pname = Getattr(ttab,k_name);
+    String *pname = Getattr(ttab,"name");
     if (Len(pname)) {
       Insert(qname,0,"::");
       Insert(qname,0,pname);
     }
-    ttab = Getattr(ttab,k_parent);
+    ttab = Getattr(ttab,"parent");
   }
   return qname;
 }
@@ -231,7 +207,7 @@ SwigType_scope_name(Typetab *ttab) {
  * Creates a new scope
  * ----------------------------------------------------------------------------- */
 
-void SwigType_new_scope(const String_or_char *name) {
+void SwigType_new_scope(String_or_char *name) {
   Typetab *s;
   Hash    *ttab;
   String  *qname;
@@ -240,15 +216,15 @@ void SwigType_new_scope(const String_or_char *name) {
     name = "<unnamed>";
   }
   s = NewHash();
-  Setattr(s,k_name, name);
-  Setattr(s,k_parent, current_scope);
+  Setattr(s,"name", name);
+  Setattr(s,"parent", current_scope);
   ttab = NewHash();
-  Setattr(s,k_typetab, ttab);
+  Setattr(s,"typetab", ttab);
 
   /* Build fully qualified name and */
   qname = SwigType_scope_name(s);
   Setattr(scopes,qname,s);
-  Setattr(s,k_qname,qname);
+  Setattr(s,"qname",qname);
   Delete(qname);
 
   current_scope = s;
@@ -268,10 +244,10 @@ void
 SwigType_inherit_scope(Typetab *scope) {
   List *inherits;
   int   i, len;
-  inherits = Getattr(current_scope,k_inherit);
+  inherits = Getattr(current_scope,"inherit");
   if (!inherits) {
     inherits = NewList();
-    Setattr(current_scope,k_inherit, inherits);
+    Setattr(current_scope,"inherit", inherits);
   }
   assert(scope != current_scope);
   
@@ -314,10 +290,10 @@ SwigType_using_scope(Typetab *scope) {
   {
     List *ulist;
     int   i, len;
-    ulist = Getattr(current_scope,k_using);
+    ulist = Getattr(current_scope,"using");
     if (!ulist) {
       ulist = NewList();
-      Setattr(current_scope,k_using, ulist);
+      Setattr(current_scope,"using", ulist);
     }
     assert(scope != current_scope);
     len = Len(ulist);
@@ -339,7 +315,7 @@ SwigType_using_scope(Typetab *scope) {
 
 Typetab *SwigType_pop_scope() {
   Typetab *s, *s1;
-  s = Getattr(current_scope,k_parent);
+  s = Getattr(current_scope,"parent");
   if (!s) {
     current_scope = 0;
     current_typetab = 0;
@@ -348,8 +324,8 @@ Typetab *SwigType_pop_scope() {
   }
   s1 = current_scope;
   current_scope = s;
-  current_typetab = Getattr(s,k_typetab);
-  current_symtab = Getattr(s,k_symtab);
+  current_typetab = Getattr(s,"typetab");
+  current_symtab = Getattr(s,"symtab");
   flush_cache();
   return s1;
 }
@@ -365,8 +341,8 @@ SwigType_set_scope(Typetab *t) {
   Typetab *old = current_scope;
   if (!t) t = global_scope;
   current_scope = t;
-  current_typetab = Getattr(t,k_typetab);
-  current_symtab = Getattr(t,k_symtab);
+  current_typetab = Getattr(t,"typetab");
+  current_symtab = Getattr(t,"symtab");
   flush_cache();
   return old;
 }
@@ -379,7 +355,7 @@ SwigType_set_scope(Typetab *t) {
 
 void
 SwigType_attach_symtab(Symtab *sym) {
-  Setattr(current_scope,k_symtab,sym);
+  Setattr(current_scope,"symtab",sym);
   current_symtab = sym;
 }
 
@@ -390,26 +366,27 @@ SwigType_attach_symtab(Symtab *sym) {
  * ----------------------------------------------------------------------------- */
 
 void SwigType_print_scope(Typetab *t) {
+  String *key;
   Hash   *ttab;
-  Iterator  i,j;
+  String *tkey;
 
-  for (i = First(scopes); i.key; i = Next(i)) {
-    t = i.item;
-    ttab = Getattr(i.item,k_typetab);
+  for (tkey = Firstkey(scopes); tkey; tkey = Nextkey(scopes)) {
+    t = Getattr(scopes,tkey);
+    ttab = Getattr(t,"typetab");
     
-    Printf(stdout,"Type scope '%s' (%x)\n", i.key, i.item);
+    Printf(stdout,"Type scope '%s' (%x)\n", tkey, t);
     {
-      List *inherit = Getattr(i.item,k_inherit);
+      List *inherit = Getattr(t,"inherit");
       if (inherit) {
-	Iterator j;
-	for (j = First(inherit); j.item; j = Next(j)) {
-	  Printf(stdout,"    Inherits from '%s' (%x)\n", Getattr(j.item,k_qname), j.item);
+	Typetab *it;
+	for (it = Firstitem(inherit); it; it = Nextitem(inherit)) {
+	  Printf(stdout,"    Inherits from '%s' (%x)\n", Getattr(it,"qname"), it);
 	}
       }
     }
     Printf(stdout,"-------------------------------------------------------------\n");
-    for (j = First(ttab); j.key; j = Next(j)) {
-      Printf(stdout,"%40s -> %s\n", j.key, j.item);
+    for (key = Firstkey(ttab); key; key = Nextkey(ttab)) {
+      Printf(stdout,"%40s -> %s\n", key, Getattr(ttab,key));
     }
   }
 }
@@ -430,7 +407,7 @@ SwigType_find_scope(Typetab *s, String *nameprefix) {
   ss = s;
   while (ss) {
     String *full;
-    String *qname = Getattr(ss,k_qname);
+    String *qname = Getattr(ss,"qname");
     if (qname) {
       full = NewStringf("%s::%s", qname, nameprefix);
     } else {
@@ -449,7 +426,7 @@ SwigType_find_scope(Typetab *s, String *nameprefix) {
     if (!s) {
       /* Check inheritance */
       List *inherit;
-      inherit = Getattr(ss,k_using);
+      inherit = Getattr(ss,"using");
       if (inherit) {
 	Typetab *ttab;
 	int i, len;
@@ -468,38 +445,35 @@ SwigType_find_scope(Typetab *s, String *nameprefix) {
       }
     }
     if (!check_parent) break;
-    ss = Getattr(ss,k_parent);
+    ss = Getattr(ss,"parent");
   }
   if (nnameprefix) Delete(nnameprefix);
   return 0;
 }
 
 /* ----------------------------------------------------------------------------- 
- * typedef_resolve()
+ * SwigType_typedef_resolve()
  *
  * Resolves a typedef and returns a new type string.  Returns 0 if there is no
  * typedef mapping.  base is a name without qualification.
- * Internal function.
  * ----------------------------------------------------------------------------- */
 
 static Typetab  *resolved_scope = 0;
 
 /* Internal function */
-
 static SwigType *
 typedef_resolve(Typetab *s, String *base) {
   Hash     *ttab;
   SwigType *type;
   List     *inherit;
-  Typetab  *parent;
 
-  /* if (!s) return 0; *//* now is checked bellow */
+  if (!s) return 0;
   /* Printf(stdout,"Typetab %s : %s\n", Getattr(s,"name"), base);  */
 
   if (Getmark(s)) return 0;
   Setmark(s,1);
 
-  ttab = Getattr(s,k_typetab);
+  ttab = Getattr(s,"typetab");
   type = Getattr(ttab,base);
   if (type) {
     resolved_scope = s;
@@ -507,7 +481,7 @@ typedef_resolve(Typetab *s, String *base) {
     return type;
   }
   /* Hmmm. Not found in my scope.  It could be in an inherited scope */
-  inherit = Getattr(s,k_inherit);
+  inherit = Getattr(s,"inherit");
   if (inherit) {
     int i,len;
     len = Len(inherit);
@@ -519,16 +493,11 @@ typedef_resolve(Typetab *s, String *base) {
       }
     }
   }
-  parent = Getattr(s,k_parent);
-  type = parent ? typedef_resolve(parent, base) : 0;
+  type = typedef_resolve(Getattr(s,"parent"), base);
   Setmark(s,0);
   return type;
 }
 
-/* ----------------------------------------------------------------------------- 
- * SwigType_typedef_resolve()
- * ----------------------------------------------------------------------------- */
-
 SwigType *SwigType_typedef_resolve(SwigType *t) {
   String *base;
   String *type = 0;
@@ -538,6 +507,7 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
   String *namebase = 0;
   String *nameprefix = 0;
   int     newtype = 0;
+  static   String *noscope = 0;
 
   /*
   if (!noscope) {
@@ -547,27 +517,31 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
 
   resolved_scope = 0;
 
-#ifdef SWIG_TYPEDEF_RESOLVE_CACHE
+  /*
   if (!typedef_resolve_cache) {
     typedef_resolve_cache = NewHash();
   }
   r = Getattr(typedef_resolve_cache,t);
   if (r) {
-    resolved_scope = Getmeta(r,k_scope);
-    return Copy(r);
+    if (r != noscope) {
+      resolved_scope = Getmeta(r,"scope");
+      return Copy(r);
+    } else {
+      return 0;
+    }
   }
-#endif
+  */
 
   base = SwigType_base(t);
 
-  /* Printf(stdout,"base = '%s' t='%s'\n", base, t); */
+  /* Printf(stdout,"base = '%s'\n", base); */
 
   if (SwigType_issimple(base)) {
     s = current_scope;
     ttab = current_typetab;
     if (Strncmp(base,"::",2) == 0) {
       s = global_scope;
-      ttab = Getattr(s,k_typetab);
+      ttab = Getattr(s,"typetab");
       Delitem(base,0);
       Delitem(base,0);
     }
@@ -581,7 +555,7 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
       if (Swig_scopename_check(base)) {
 	/* A qualified name. */
 	nameprefix = Swig_scopename_prefix(base);
-	/* Printf(stdout,"nameprefix = '%s'\n", nameprefix); */
+	/*	Printf(stdout,"nameprefix = '%s'\n", nameprefix);*/
 	if (nameprefix) {
 	  /* Name had a prefix on it.   See if we can locate the proper scope for it */
 	  s = SwigType_find_scope(s,nameprefix);
@@ -600,7 +574,7 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
 	  /* Printf(stdout,"%s type = '%s'\n", Getattr(s,"name"), type); */
 	  if ((type) && (!Swig_scopename_check(type))) {
 	    Typetab *rtab = resolved_scope;
-	    String *qname = Getattr(resolved_scope,k_qname);
+	    String *qname = Getattr(resolved_scope,"qname");
 	    /* If qualified *and* the typename is defined from the resolved scope, we qualify */
 	    if ((qname) && typedef_resolve(resolved_scope,type)) {
 	      type = Copy(type);
@@ -745,20 +719,22 @@ SwigType *SwigType_typedef_resolve(SwigType *t) {
   }
 
  return_result:
-#ifdef SWIG_TYPEDEF_RESOLVE_CACHE
+  /*
   {
     String *key = NewString(t);
-    if (r) {
+    if (!r) {
+      Setattr(typedef_resolve_cache,key,noscope);
+    } else {
       SwigType *r1;
       Setattr(typedef_resolve_cache,key,r);
-      Setmeta(r,k_scope,resolved_scope);
+      Setmeta(r,"scope",resolved_scope);
       r1 = Copy(r);
       Delete(r);
       r = r1;
     }
     Delete(key);
   }
-#endif
+  */
   return r;
 }
 
@@ -772,7 +748,6 @@ SwigType *SwigType_typedef_resolve_all(SwigType *t) {
   SwigType *n;
   SwigType *r;
 
-  /* Check to see if the typedef resolve has been done before by checking the cache */
   if (!typedef_all_cache) {
     typedef_all_cache = NewHash();
   }
@@ -780,15 +755,11 @@ SwigType *SwigType_typedef_resolve_all(SwigType *t) {
   if (r) {
     return Copy(r);
   }
-
-  /* Recursively resolve the typedef */
   r = NewString(t);
   while ((n = SwigType_typedef_resolve(r))) {
     Delete(r);
     r = n;
   }
-
-  /* Add the typedef to the cache for next time it is looked up */
   {
     String *key;
     SwigType *rr = Copy(r);
@@ -813,13 +784,13 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
   List   *elements;
   String *result;
   int     i,len;
-  if (!typedef_qualified_cache) typedef_qualified_cache = NewHash();
+
+ if (!typedef_qualified_cache) typedef_qualified_cache = NewHash();
   result = Getattr(typedef_qualified_cache,t);
   if (result) {
     String *rc = Copy(result);
     return rc;
   }
-
   result = NewString("");
   elements = SwigType_split(t);
   len = Len(elements);
@@ -829,18 +800,24 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
       if (!SwigType_istemplate(e)) {
 	String *isenum = 0;
 	if (SwigType_isenum(e)) {
-	  isenum = NewString("enum ");
+	  isenum = e;
 	  e = NewString(Char(e)+5);
 	}
 	resolved_scope = 0;
 	if (typedef_resolve(current_scope,e)) {
 	  /* resolved_scope contains the scope that actually resolved the symbol */
-	  String *qname = Getattr(resolved_scope,k_qname);
+	  String *qname = Getattr(resolved_scope,"qname");
 	  if (qname) {
 	    Insert(e,0,"::");
 	    Insert(e,0,qname);
+	    if (isenum) {
+	      Clear(isenum);
+	      Printf(isenum, "enum %s", e);
+	      Delete(e);
+	    }
 	  }
 	} else {
+
 	  if (Swig_scopename_check(e)) {
 	    String *tqname;
 	    String *qlast;
@@ -855,49 +832,25 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	      Delete(tqname);
 	    }
 	    /* Automatic template instantiation might go here??? */
-	  } else {
-	    /* It's a bare name.  It's entirely possible, that the
-               name is part of a namespace. We'll check this by unrolling
-               out of the current scope */
-	     
-	    Typetab *cs = current_scope;
-	    while (cs) {
-	      String *qs = SwigType_scope_name(cs);
-	      if (Len(qs)) {
-		Append(qs,"::");
-	      }
-	      Append(qs,e);
-	      if (Getattr(scopes,qs)) {
-		Clear(e);
-		Append(e,qs);
-		Delete(qs);
-		break;
-	      }
-	      Delete(qs);
-	      cs = Getattr(cs,k_parent);
-	    }
+
 	  }
 	}
-
-	if (isenum) {
-          Insert(e,0,isenum);
-        }
+	if (isenum) e = isenum;
 
       } else {
 	/* Template.  We need to qualify template parameters as well as the template itself */
 	String *tprefix, *qprefix;
 	String *tsuffix;
-	Iterator pi;
 	Parm   *p;
 	List *parms = SwigType_parmlist(e);
 	tprefix = SwigType_templateprefix(e);
 	tsuffix = SwigType_templatesuffix(e);
 	qprefix = SwigType_typedef_qualified(tprefix);
-	Printv(qprefix,"<(",NIL);
-	pi = First(parms);
-	while ((p = pi.item)) {
+	Printf(qprefix,"<(");
+	p = Firstitem(parms);
+	while (p) {
 	  String *qt = SwigType_typedef_qualified(p);
-	  if ((Strcmp(qt,p) == 0)) { /*  && (!Swig_scopename_check(qt))) */
+	  if ((Strcmp(qt,p) == 0)) { /*  && (!Swig_scopename_check(qt))) { */
 	    /* No change in value.  It is entirely possible that the parameter is an integer value.
 	       If there is a symbol table associated with this scope, we're going to check for this */
 
@@ -914,7 +867,7 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 		    String *qn = Swig_symbol_qualified(n);
 		    if (Len(qn)) {
 		      Append(qn,"::");
-		      Append(qn,Getattr(n,k_name));
+		      Append(qn,Getattr(n,"name"));
 		      Delete(value);
 		      value = qn;
 		      continue;
@@ -922,9 +875,9 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 		      Delete(qn);
 		      break;
 		    }
-		  } else if ((Strcmp(nodeType(n),"cdecl") == 0) && (Getattr(n,k_value))) {
+		  } else if ((Strcmp(nodeType(n),"cdecl") == 0) && (Getattr(n,"value"))) {
 		    Delete(value);
-		    value = Copy(Getattr(n,k_value));
+		    value = Copy(Getattr(n,"value"));
 		    continue;
 		  }
 		}
@@ -939,8 +892,8 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
 	    Append(qprefix,qt);
 	  }
 	  Delete(qt);
-	  pi= Next(pi);
-	  if (pi.item) {
+	  p= Nextitem(parms);
+	  if (p) {
 	    Append(qprefix,",");
 	  }
 	}
@@ -961,14 +914,14 @@ SwigType *SwigType_typedef_qualified(SwigType *t)
     } else if (SwigType_isfunction(e)) {
       List *parms = SwigType_parmlist(e);
       String *s = NewString("f(");
-      Iterator pi;
-      pi = First(parms);
-      while (pi.item) {
-	String *pq = SwigType_typedef_qualified(pi.item);
+      String *p;
+      p = Firstitem(parms);
+      while (p) {
+	String *pq = SwigType_typedef_qualified(p);
 	Append(s,pq);
 	Delete(pq);
-	pi = Next(pi);
-	if (pi.item) {
+	p = Nextitem(parms);
+	if (p) {
 	  Append(s,",");
 	}
       }
@@ -1056,7 +1009,7 @@ int SwigType_typedef_using(String_or_char *name) {
   td = SwigType_typedef_resolve(name);
   /*  Printf(stdout,"td = '%s' %x\n", td, resolved_scope); */
   if (resolved_scope) {
-    defined_name = Getattr(resolved_scope,k_qname);
+    defined_name = Getattr(resolved_scope,"qname");
     if (defined_name) {
       defined_name = Copy(defined_name);
       Append(defined_name,"::");
@@ -1073,7 +1026,7 @@ int SwigType_typedef_using(String_or_char *name) {
     prefix = Swig_scopename_prefix(name);
     s = SwigType_find_scope(current_scope,prefix);
     if (s) {
-      Hash *ttab = Getattr(s,k_typetab);
+      Hash *ttab = Getattr(s,"typetab");
       if (!Getattr(ttab,base) && defined_name) {
 	Setattr(ttab,base, defined_name);
       }
@@ -1117,11 +1070,9 @@ SwigType_isclass(SwigType *t) {
       isclass = 1;
     }
     /* Hmmm. Not a class.  If a template, it might be uninstantiated */
-    if (!isclass && SwigType_istemplate(qtys)) {
+    if (SwigType_istemplate(qtys)) {
       String *tp = SwigType_templateprefix(qtys);
-      if (Strcmp(tp,t) != 0) {
-	isclass = SwigType_isclass(tp);
-      }
+      isclass = SwigType_isclass(tp);
       Delete(tp);
     }
   }
@@ -1276,11 +1227,10 @@ void SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata)
     (*r_tracefunc)(t,mt, (String *) clientdata);
   }
 
-  if (SwigType_istypedef(t)) {
+  if (SwigType_istypedef(t))
     lt = Copy(t);
-  } else {
+  else
     lt = SwigType_ltype(t);
-  }
   Setattr(r_ltype, mt, lt);
   fr = SwigType_typedef_resolve_all(t);     /* Create fully resolved type */
   qr = SwigType_typedef_qualified(fr);
@@ -1326,17 +1276,6 @@ void SwigType_remember_clientdata(SwigType *t, const String_or_char *clientdata)
       Setattr(r_clientdata, fr, NewString(clientdata));
     }
   }
-
-  /* If the remembered type is a reference, we also remember the pointer version.
-     This is to prevent odd problems with mixing pointers and references--especially
-     when different functions are using different typenames (via typedef). */
-
-  if (SwigType_isreference(t)) {
-    SwigType *tt = Copy(t);
-    SwigType_del_reference(tt);
-    SwigType_add_pointer(tt);
-    SwigType_remember_clientdata(tt,clientdata);
-  }
 }
 
 void
@@ -1380,26 +1319,26 @@ List *SwigType_equivalent_mangle(String *ms, Hash *checked, Hash *found) {
   Setattr(ch, ms, "1");
   mh = Getattr(r_mangled,ms);
   if (mh) {
-    Iterator ki;
-    ki = First(mh);
-    while (ki.key) {
+    String *key;
+    key = Firstkey(mh);
+    while (key) {
       Hash *rh;
-      if (Getattr(ch,ki.key)) {
-	ki = Next(ki);
+      if (Getattr(ch,key)) {
+	key = Nextkey(mh);
 	continue;
       }
-      Setattr(ch,ki.key,"1");
-      rh = Getattr(r_resolved,ki.key);
+      Setattr(ch,key,"1");
+      rh = Getattr(r_resolved,key);
       if (rh) {
-	Iterator rk;
-	rk = First(rh);
-	while (rk.key) {
-	  Setattr(h,rk.key,"1");
-	  SwigType_equivalent_mangle(rk.key,ch,h);
-	  rk = Next(rk);
+	String *rkey;
+	rkey = Firstkey(rh);
+	while (rkey) {
+	  Setattr(h,rkey,"1");
+	  SwigType_equivalent_mangle(rkey,ch,h);
+	  rkey = Nextkey(rh);
 	}
       }
-      ki = Next(ki);
+      key = Nextkey(mh);
     }
   }
  check_exit:
@@ -1434,24 +1373,24 @@ String *SwigType_clientdata_collect(String *ms, Hash *checked) {
   Setattr(ch, ms, "1");
   mh = Getattr(r_mangled,ms);
   if (mh) {
-    Iterator ki;
-    ki = First(mh);
-    while (ki.key) {
+    String *key;
+    key = Firstkey(mh);
+    while (key) {
       Hash *rh;
-      Setattr(ch,ki.key,"1");
-      clientdata = Getattr(r_clientdata,ki.key);
+      Setattr(ch,key,"1");
+      clientdata = Getattr(r_clientdata,key);
       if (clientdata) goto check_exit;
-      rh = Getattr(r_resolved,ki.key);
+      rh = Getattr(r_resolved,key);
       if (rh) {
-	Iterator rk;
-	rk = First(rh);
-	while (rk.key) {
-	  clientdata = SwigType_clientdata_collect(rk.key,ch);
+	String *rkey;
+	rkey = Firstkey(rh);
+	while (rkey) {
+	  clientdata = SwigType_clientdata_collect(rkey,ch);
 	  if (clientdata) goto check_exit;
-	  rk = Next(rk);
+	  rkey = Nextkey(rh);
 	}
       }
-      ki = Next(ki);
+      key = Nextkey(mh);
     }
   }
  check_exit:
@@ -1545,49 +1484,48 @@ SwigType_issubtype(SwigType *t1, SwigType *t2) {
  * ----------------------------------------------------------------------------- */
 
 void SwigType_inherit_equiv(File *out) {
-  String *ckey;
+  String *rkey, *bkey, *ckey;
   String *prefix, *base;
   Hash   *sub;
   Hash   *rh;
   List   *rlist;
-  Iterator rk, bk, ck;
 
   if (!conversions) conversions = NewHash();
   if (!subclass) subclass = NewHash();
 
-  rk = First(r_resolved);
-  while (rk.key) {
+  rkey = Firstkey(r_resolved);
+  while (rkey) {
     /* rkey is a fully qualified type.  We strip all of the type constructors off of it just to get the base */
-    base = SwigType_base(rk.key);
+    base = SwigType_base(rkey);
     /* Check to see whether the base is recorded in the subclass table */
     sub = Getattr(subclass,base);
     Delete(base);
     if (!sub) {
-      rk = Next(rk);
+      rkey = Nextkey(r_resolved);
       continue;
     }
 
     /* This type has subclasses.  We now need to walk through these subtypes and generate pointer converion functions */
 
-    rh = Getattr(r_resolved, rk.key);
+    rh = Getattr(r_resolved, rkey);
     rlist = NewList();
-    for (ck = First(rh); ck.key; ck = Next(ck)) {
-      Append(rlist,ck.key);
+    for (ckey = Firstkey(rh); ckey; ckey = Nextkey(rh)) {
+      Append(rlist,ckey);
     }
-    /*    Printf(stdout,"rk.key = '%s'\n", rk.key);
+    /*    Printf(stdout,"rkey = '%s'\n", rkey);
 	  Printf(stdout,"rh = %x '%s'\n", rh,rh); */
 
-    bk = First(sub);
-    while (bk.key) {
-      prefix= SwigType_prefix(rk.key);
-      Append(prefix,bk.key);
+    bkey = Firstkey(sub);
+    while (bkey) {
+      prefix= SwigType_prefix(rkey);
+      Append(prefix,bkey);
       /*      Printf(stdout,"set %x = '%s' : '%s'\n", rh, SwigType_manglestr(prefix),prefix); */
       Setattr(rh,SwigType_manglestr(prefix),prefix);
-      ckey = NewStringf("%s+%s",SwigType_manglestr(prefix), SwigType_manglestr(rk.key));
+      ckey = NewStringf("%s+%s",SwigType_manglestr(prefix), SwigType_manglestr(rkey));
       if (!Getattr(conversions,ckey)) {
-	String *convname = NewStringf("%sTo%s", SwigType_manglestr(prefix), SwigType_manglestr(rk.key));
+	String *convname = NewStringf("%sTo%s", SwigType_manglestr(prefix), SwigType_manglestr(rkey));
 	Printf(out,"static void *%s(void *x) {\n", convname);
-	Printf(out,"    return (void *)((%s) %s ((%s) x));\n", SwigType_lstr(rk.key,0), Getattr(sub,bk.key), SwigType_lstr(prefix,0));
+	Printf(out,"    return (void *)((%s) %s ((%s) x));\n", SwigType_lstr(rkey,0), Getattr(sub,bkey), SwigType_lstr(prefix,0));
 	Printf(out,"}\n");
 	Setattr(conversions,ckey,convname);
 	Delete(ckey);	
@@ -1596,40 +1534,39 @@ void SwigType_inherit_equiv(File *out) {
 	{
 	  Hash *r = Getattr(r_resolved, prefix);
 	  if (r) {
-	    Iterator rrk;
-	    rrk=First(r);
-	    while (rrk.key) {
-	      Iterator rlk;
+	    String *rrkey = Firstkey(r);
+	    while (rrkey) {
+	      String *rlkey;
 	      String *rkeymangle;
 
 	      /* Make sure this name equivalence is not due to inheritance */
-	      if (Cmp(prefix, Getattr(r,rrk.key)) == 0) {
-		rkeymangle = SwigType_manglestr(rk.key);
-		ckey = NewStringf("%s+%s", rrk.key, rkeymangle);
+	      if (Cmp(prefix, Getattr(r,rrkey)) == 0) {
+		rkeymangle = SwigType_manglestr(rkey);
+		ckey = NewStringf("%s+%s", rrkey, rkeymangle);
 		if (!Getattr(conversions, ckey)) {
 		  Setattr(conversions, ckey, convname);
 		}
 		Delete(ckey);
-		for (rlk = First(rlist); rlk.item; rlk = Next(rlk)) {
-		  ckey = NewStringf("%s+%s", rrk.key, rlk.item);
+		for (rlkey = Firstitem(rlist); rlkey; rlkey = Nextitem(rlist)) {
+		  ckey = NewStringf("%s+%s", rrkey, rlkey);
 		  Setattr(conversions, ckey, convname);
 		  Delete(ckey);
 		}
 		Delete(rkeymangle);
 		/* This is needed to pick up other alternative names for the same type.
                    Needed to make templates work */
-	        Setattr(rh,rrk.key,rrk.item);   
+	        Setattr(rh,rrkey,Getattr(r,rrkey));   
 	      }
-	      rrk = Next(rrk);
+	      rrkey = Nextkey(r);
 	    }
 	  }
 	}
 	Delete(convname);
       }
       Delete(prefix);
-      bk = Next(bk);
+      bkey = Nextkey(sub);
     }
-    rk = Next(rk);
+    rkey = Nextkey(r_resolved);
   }
 }
 
@@ -1641,7 +1578,7 @@ void SwigType_inherit_equiv(File *out) {
 
 void
 SwigType_emit_type_table(File *f_forward, File *f_table) {
-  Iterator ki;
+  DOH *key;
   String *types, *table;
   int i = 0;
 
@@ -1675,54 +1612,36 @@ SwigType_emit_type_table(File *f_forward, File *f_table) {
   table = NewString("");
   types = NewString("");
   Printf(table,"static swig_type_info *swig_types_initial[] = {\n");
-
-  ki = First(r_mangled);
+  key = Firstkey(r_mangled);
   Printf(f_forward,"\n/* -------- TYPES TABLE (BEGIN) -------- */\n\n");
-  while (ki.key) {
+  while (key) {
     List *el;
-    Iterator ei;
-    SwigType *lt;
-    SwigType *rt;
-    String *nt;
-    String *ln;
-    String *rn;
-    const String *cd;
+    String *en;
+    String *cd;
 
-    Printf(f_forward,"#define  SWIGTYPE%s swig_types[%d] \n", ki.key, i);
-    Printv(types,"static swig_type_info _swigt_", ki.key, "[] = {", NIL);
+    Printf(f_forward,"#define  SWIGTYPE%s swig_types[%d] \n", key, i);
+    Printv(types,"static swig_type_info _swigt_", key, "[] = {", NIL);
     
-    cd = SwigType_clientdata_collect(ki.key,0);
+    cd = SwigType_clientdata_collect(key,0);
     if (!cd) cd = "0";
-    lt = Getattr(r_ltype,ki.key);
-    rt = SwigType_typedef_resolve_all(lt);
-    /* we save the original type and the fully resolved version */
-    ln = SwigType_str(lt,0);
-    rn = SwigType_str(rt,0);
-    if (Strcmp(ln,rn) == 0) {
-      nt = NewStringf("%s", ln);
-    } else {
-      nt = NewStringf("%s|%s", rn, ln);
-    }
-    Printv(types,"{\"", ki.key, "\", 0, \"",nt,"\", ", cd, ", 0, 0, 0},", NIL);
-    el = SwigType_equivalent_mangle(ki.key,0,0);
-    for (ei = First(el); ei.item; ei = Next(ei)) {
+    Printv(types,"{\"", key, "\", 0, \"", SwigType_str(Getattr(r_ltype,key),0),"\", ", cd, "},", NIL);
+    el = SwigType_equivalent_mangle(key,0,0);
+    for (en = Firstitem(el); en; en = Nextitem(el)) {
       String *ckey;
       String *conv;
-      ckey = NewStringf("%s+%s", ei.item, ki.key);
+      ckey = NewStringf("%s+%s", en, key);
       conv = Getattr(conversions,ckey);
       if (conv) {
-	Printf(types,"{\"%s\", %s, 0, 0, 0, 0, 0},", ei.item, conv);
+	Printf(types,"{\"%s\", %s},", en, conv);
       } else {
-	Printf(types,"{\"%s\", 0, 0, 0, 0, 0, 0},", ei.item);
+	Printf(types,"{\"%s\"},", en);
       }
       Delete(ckey);
     }
     Delete(el);
-    Delete(nt);
-    Delete(rt);
-    Printf(types,"{0, 0, 0, 0, 0, 0, 0}};\n");
-    Printv(table, "_swigt_", ki.key, ", \n", NIL);
-    ki = Next(ki);
+    Printf(types,"{0}};\n");
+    Printv(table, "_swigt_", key, ", \n", NIL);
+    key = Nextkey(r_mangled);
     i++;
   }
 
